// Copyright (c) Microsoft Corporation
// All rights reserved. 
//
// Licensed under the Apache License, Version 2.0 (the ""License""); you may not use this file except in compliance with the License. You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 
//
// THIS CODE IS PROVIDED ON AN  *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE, MERCHANTABLITY OR NON-INFRINGEMENT. 
//
// See the Apache Version 2.0 License for specific language governing permissions and limitations under the License.

#define NTDDI_VERSION 0x06010000

#include <v8.h>
#include <string>
#include <node_object_wrap.h>
#include <ppltasks.h>
#include "CollectionsConverter.h"
#include "CollectionsWrap.h"
#include "node-async.h"
#include "NodeRtUtils.h"
#include "OpaqueWrapper.h"
#include "WrapperBase.h"

#using <Windows.WinMD>

// this undefs fixes the issues of compiling Windows.Data.Json, Windows.Storag.FileProperties, and Windows.Stroage.Search
// Some of the node header files brings windows definitions with the same names as some of the WinRT methods
#undef DocumentProperties
#undef GetObject
#undef CreateEvent
#undef FindText
#undef SendMessage

const char* REGISTRATION_TOKEN_MAP_PROPERTY_NAME = "__registrationTokenMap__";

using namespace v8;
using namespace node;
using namespace concurrency;

namespace NodeRT { namespace Windows { namespace Foundation { namespace Diagnostics { 
  v8::Handle<v8::Value> WrapTracingStatusChangedEventArgs(::Windows::Foundation::Diagnostics::TracingStatusChangedEventArgs^ wintRtInstance);
  ::Windows::Foundation::Diagnostics::TracingStatusChangedEventArgs^ UnwrapTracingStatusChangedEventArgs(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapAsyncCausalityTracer(::Windows::Foundation::Diagnostics::AsyncCausalityTracer^ wintRtInstance);
  ::Windows::Foundation::Diagnostics::AsyncCausalityTracer^ UnwrapAsyncCausalityTracer(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapIErrorReportingSettings(::Windows::Foundation::Diagnostics::IErrorReportingSettings^ wintRtInstance);
  ::Windows::Foundation::Diagnostics::IErrorReportingSettings^ UnwrapIErrorReportingSettings(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapRuntimeBrokerErrorSettings(::Windows::Foundation::Diagnostics::RuntimeBrokerErrorSettings^ wintRtInstance);
  ::Windows::Foundation::Diagnostics::RuntimeBrokerErrorSettings^ UnwrapRuntimeBrokerErrorSettings(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapILoggingChannel(::Windows::Foundation::Diagnostics::ILoggingChannel^ wintRtInstance);
  ::Windows::Foundation::Diagnostics::ILoggingChannel^ UnwrapILoggingChannel(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapLoggingChannel(::Windows::Foundation::Diagnostics::LoggingChannel^ wintRtInstance);
  ::Windows::Foundation::Diagnostics::LoggingChannel^ UnwrapLoggingChannel(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapLoggingActivity(::Windows::Foundation::Diagnostics::LoggingActivity^ wintRtInstance);
  ::Windows::Foundation::Diagnostics::LoggingActivity^ UnwrapLoggingActivity(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapILoggingSession(::Windows::Foundation::Diagnostics::ILoggingSession^ wintRtInstance);
  ::Windows::Foundation::Diagnostics::ILoggingSession^ UnwrapILoggingSession(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapLoggingSession(::Windows::Foundation::Diagnostics::LoggingSession^ wintRtInstance);
  ::Windows::Foundation::Diagnostics::LoggingSession^ UnwrapLoggingSession(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapLogFileGeneratedEventArgs(::Windows::Foundation::Diagnostics::LogFileGeneratedEventArgs^ wintRtInstance);
  ::Windows::Foundation::Diagnostics::LogFileGeneratedEventArgs^ UnwrapLogFileGeneratedEventArgs(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapIFileLoggingSession(::Windows::Foundation::Diagnostics::IFileLoggingSession^ wintRtInstance);
  ::Windows::Foundation::Diagnostics::IFileLoggingSession^ UnwrapIFileLoggingSession(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapFileLoggingSession(::Windows::Foundation::Diagnostics::FileLoggingSession^ wintRtInstance);
  ::Windows::Foundation::Diagnostics::FileLoggingSession^ UnwrapFileLoggingSession(Handle<Value> value);
  


  static v8::Handle<v8::Value> InitCausalityTraceLevelEnum(const Handle<Object> exports)
  {
    HandleScope scope;
    
    Handle<Object> enumObject = Object::New();
    exports->Set(String::NewSymbol("CausalityTraceLevel"), enumObject);

    enumObject->Set(String::NewSymbol("required"), Integer::New(0));
    enumObject->Set(String::NewSymbol("important"), Integer::New(1));
    enumObject->Set(String::NewSymbol("verbose"), Integer::New(2));

    return scope.Close(Undefined());
  }


  static v8::Handle<v8::Value> InitCausalitySourceEnum(const Handle<Object> exports)
  {
    HandleScope scope;
    
    Handle<Object> enumObject = Object::New();
    exports->Set(String::NewSymbol("CausalitySource"), enumObject);

    enumObject->Set(String::NewSymbol("application"), Integer::New(0));
    enumObject->Set(String::NewSymbol("library"), Integer::New(1));
    enumObject->Set(String::NewSymbol("system"), Integer::New(2));

    return scope.Close(Undefined());
  }


  static v8::Handle<v8::Value> InitCausalityRelationEnum(const Handle<Object> exports)
  {
    HandleScope scope;
    
    Handle<Object> enumObject = Object::New();
    exports->Set(String::NewSymbol("CausalityRelation"), enumObject);

    enumObject->Set(String::NewSymbol("assignDelegate"), Integer::New(0));
    enumObject->Set(String::NewSymbol("join"), Integer::New(1));
    enumObject->Set(String::NewSymbol("choice"), Integer::New(2));
    enumObject->Set(String::NewSymbol("cancel"), Integer::New(3));
    enumObject->Set(String::NewSymbol("error"), Integer::New(4));

    return scope.Close(Undefined());
  }


  static v8::Handle<v8::Value> InitCausalitySynchronousWorkEnum(const Handle<Object> exports)
  {
    HandleScope scope;
    
    Handle<Object> enumObject = Object::New();
    exports->Set(String::NewSymbol("CausalitySynchronousWork"), enumObject);

    enumObject->Set(String::NewSymbol("completionNotification"), Integer::New(0));
    enumObject->Set(String::NewSymbol("progressNotification"), Integer::New(1));
    enumObject->Set(String::NewSymbol("execution"), Integer::New(2));

    return scope.Close(Undefined());
  }


  static v8::Handle<v8::Value> InitErrorOptionsEnum(const Handle<Object> exports)
  {
    HandleScope scope;
    
    Handle<Object> enumObject = Object::New();
    exports->Set(String::NewSymbol("ErrorOptions"), enumObject);

    enumObject->Set(String::NewSymbol("none"), Integer::New(0));
    enumObject->Set(String::NewSymbol("suppressExceptions"), Integer::New(1));
    enumObject->Set(String::NewSymbol("forceExceptions"), Integer::New(2));
    enumObject->Set(String::NewSymbol("useSetErrorInfo"), Integer::New(3));
    enumObject->Set(String::NewSymbol("suppressSetErrorInfo"), Integer::New(4));

    return scope.Close(Undefined());
  }


  static v8::Handle<v8::Value> InitLoggingLevelEnum(const Handle<Object> exports)
  {
    HandleScope scope;
    
    Handle<Object> enumObject = Object::New();
    exports->Set(String::NewSymbol("LoggingLevel"), enumObject);

    enumObject->Set(String::NewSymbol("verbose"), Integer::New(0));
    enumObject->Set(String::NewSymbol("information"), Integer::New(1));
    enumObject->Set(String::NewSymbol("warning"), Integer::New(2));
    enumObject->Set(String::NewSymbol("error"), Integer::New(3));
    enumObject->Set(String::NewSymbol("critical"), Integer::New(4));

    return scope.Close(Undefined());
  }



  
  class TracingStatusChangedEventArgs : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("TracingStatusChangedEventArgs"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
                              
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("enabled"), EnabledGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("traceLevel"), TraceLevelGetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("TracingStatusChangedEventArgs"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    TracingStatusChangedEventArgs(::Windows::Foundation::Diagnostics::TracingStatusChangedEventArgs^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Foundation::Diagnostics::TracingStatusChangedEventArgs^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::TracingStatusChangedEventArgs^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Foundation::Diagnostics::TracingStatusChangedEventArgs^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      TracingStatusChangedEventArgs *wrapperInstance = new TracingStatusChangedEventArgs(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  



    static Handle<Value> EnabledGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::TracingStatusChangedEventArgs^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      TracingStatusChangedEventArgs *wrapper = TracingStatusChangedEventArgs::Unwrap<TracingStatusChangedEventArgs>(info.This());

      try 
      {
        bool result = wrapper->_instance->Enabled;
        return scope.Close(Boolean::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> TraceLevelGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::TracingStatusChangedEventArgs^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      TracingStatusChangedEventArgs *wrapper = TracingStatusChangedEventArgs::Unwrap<TracingStatusChangedEventArgs>(info.This());

      try 
      {
        ::Windows::Foundation::Diagnostics::CausalityTraceLevel result = wrapper->_instance->TraceLevel;
        return scope.Close(Integer::New(static_cast<int>(result)));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    


  private:
    ::Windows::Foundation::Diagnostics::TracingStatusChangedEventArgs^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapTracingStatusChangedEventArgs(::Windows::Foundation::Diagnostics::TracingStatusChangedEventArgs^ wintRtInstance);
    friend ::Windows::Foundation::Diagnostics::TracingStatusChangedEventArgs^ UnwrapTracingStatusChangedEventArgs(Handle<Value> value);
    friend bool IsTracingStatusChangedEventArgsWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> TracingStatusChangedEventArgs::s_constructorTemplate;

  v8::Handle<v8::Value> WrapTracingStatusChangedEventArgs(::Windows::Foundation::Diagnostics::TracingStatusChangedEventArgs^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(TracingStatusChangedEventArgs::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Foundation::Diagnostics::TracingStatusChangedEventArgs^ UnwrapTracingStatusChangedEventArgs(Handle<Value> value)
  {
     return TracingStatusChangedEventArgs::Unwrap<TracingStatusChangedEventArgs>(value.As<Object>())->_instance;
  }

  void InitTracingStatusChangedEventArgs(Handle<Object> exports)
  {
    TracingStatusChangedEventArgs::Init(exports);
  }

  class AsyncCausalityTracer : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("AsyncCausalityTracer"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
                              
      Local<Function> constructor = s_constructorTemplate->GetFunction();

      constructor->Set(String::NewSymbol("traceOperationCreation"), FunctionTemplate::New(TraceOperationCreation)->GetFunction());
      constructor->Set(String::NewSymbol("traceOperationCompletion"), FunctionTemplate::New(TraceOperationCompletion)->GetFunction());
      constructor->Set(String::NewSymbol("traceOperationRelation"), FunctionTemplate::New(TraceOperationRelation)->GetFunction());
      constructor->Set(String::NewSymbol("traceSynchronousWorkStart"), FunctionTemplate::New(TraceSynchronousWorkStart)->GetFunction());
      constructor->Set(String::NewSymbol("traceSynchronousWorkCompletion"), FunctionTemplate::New(TraceSynchronousWorkCompletion)->GetFunction());
      
      Local<Function> addListenerFunc = FunctionTemplate::New(AddListener)->GetFunction();
      constructor->Set(String::NewSymbol("addListener"), addListenerFunc);
      constructor->Set(String::NewSymbol("on"), addListenerFunc);
      Local<Function> removeListenerFunc = FunctionTemplate::New(RemoveListener)->GetFunction();
      constructor->Set(String::NewSymbol("removeListener"), removeListenerFunc);
      constructor->Set(String::NewSymbol("off"), removeListenerFunc);

      exports->Set(String::NewSymbol("AsyncCausalityTracer"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    AsyncCausalityTracer(::Windows::Foundation::Diagnostics::AsyncCausalityTracer^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Foundation::Diagnostics::AsyncCausalityTracer^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::AsyncCausalityTracer^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Foundation::Diagnostics::AsyncCausalityTracer^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      AsyncCausalityTracer *wrapperInstance = new AsyncCausalityTracer(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  


    static Handle<Value> TraceOperationCreation(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() == 6
        && args[0]->IsInt32()
        && args[1]->IsInt32()
        && NodeRT::Utils::IsGuid(args[2])
        && args[3]->IsNumber()
        && args[4]->IsString()
        && args[5]->IsNumber())
      {
        try
        {
          ::Windows::Foundation::Diagnostics::CausalityTraceLevel arg0 = static_cast<::Windows::Foundation::Diagnostics::CausalityTraceLevel>(args[0]->Int32Value());
          ::Windows::Foundation::Diagnostics::CausalitySource arg1 = static_cast<::Windows::Foundation::Diagnostics::CausalitySource>(args[1]->Int32Value());
          ::Platform::Guid arg2 = NodeRT::Utils::GuidFromJs(args[2]);
          unsigned __int64 arg3 = static_cast<unsigned __int64>(args[3]->IntegerValue());
          Platform::String^ arg4 = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(args[4])));
          unsigned __int64 arg5 = static_cast<unsigned __int64>(args[5]->IntegerValue());
          
          ::Windows::Foundation::Diagnostics::AsyncCausalityTracer::TraceOperationCreation(arg0, arg1, arg2, arg3, arg4, arg5);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> TraceOperationCompletion(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() == 5
        && args[0]->IsInt32()
        && args[1]->IsInt32()
        && NodeRT::Utils::IsGuid(args[2])
        && args[3]->IsNumber()
        && args[4]->IsInt32())
      {
        try
        {
          ::Windows::Foundation::Diagnostics::CausalityTraceLevel arg0 = static_cast<::Windows::Foundation::Diagnostics::CausalityTraceLevel>(args[0]->Int32Value());
          ::Windows::Foundation::Diagnostics::CausalitySource arg1 = static_cast<::Windows::Foundation::Diagnostics::CausalitySource>(args[1]->Int32Value());
          ::Platform::Guid arg2 = NodeRT::Utils::GuidFromJs(args[2]);
          unsigned __int64 arg3 = static_cast<unsigned __int64>(args[3]->IntegerValue());
          ::Windows::Foundation::AsyncStatus arg4 = static_cast<::Windows::Foundation::AsyncStatus>(args[4]->Int32Value());
          
          ::Windows::Foundation::Diagnostics::AsyncCausalityTracer::TraceOperationCompletion(arg0, arg1, arg2, arg3, arg4);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> TraceOperationRelation(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() == 5
        && args[0]->IsInt32()
        && args[1]->IsInt32()
        && NodeRT::Utils::IsGuid(args[2])
        && args[3]->IsNumber()
        && args[4]->IsInt32())
      {
        try
        {
          ::Windows::Foundation::Diagnostics::CausalityTraceLevel arg0 = static_cast<::Windows::Foundation::Diagnostics::CausalityTraceLevel>(args[0]->Int32Value());
          ::Windows::Foundation::Diagnostics::CausalitySource arg1 = static_cast<::Windows::Foundation::Diagnostics::CausalitySource>(args[1]->Int32Value());
          ::Platform::Guid arg2 = NodeRT::Utils::GuidFromJs(args[2]);
          unsigned __int64 arg3 = static_cast<unsigned __int64>(args[3]->IntegerValue());
          ::Windows::Foundation::Diagnostics::CausalityRelation arg4 = static_cast<::Windows::Foundation::Diagnostics::CausalityRelation>(args[4]->Int32Value());
          
          ::Windows::Foundation::Diagnostics::AsyncCausalityTracer::TraceOperationRelation(arg0, arg1, arg2, arg3, arg4);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> TraceSynchronousWorkStart(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() == 5
        && args[0]->IsInt32()
        && args[1]->IsInt32()
        && NodeRT::Utils::IsGuid(args[2])
        && args[3]->IsNumber()
        && args[4]->IsInt32())
      {
        try
        {
          ::Windows::Foundation::Diagnostics::CausalityTraceLevel arg0 = static_cast<::Windows::Foundation::Diagnostics::CausalityTraceLevel>(args[0]->Int32Value());
          ::Windows::Foundation::Diagnostics::CausalitySource arg1 = static_cast<::Windows::Foundation::Diagnostics::CausalitySource>(args[1]->Int32Value());
          ::Platform::Guid arg2 = NodeRT::Utils::GuidFromJs(args[2]);
          unsigned __int64 arg3 = static_cast<unsigned __int64>(args[3]->IntegerValue());
          ::Windows::Foundation::Diagnostics::CausalitySynchronousWork arg4 = static_cast<::Windows::Foundation::Diagnostics::CausalitySynchronousWork>(args[4]->Int32Value());
          
          ::Windows::Foundation::Diagnostics::AsyncCausalityTracer::TraceSynchronousWorkStart(arg0, arg1, arg2, arg3, arg4);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> TraceSynchronousWorkCompletion(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() == 3
        && args[0]->IsInt32()
        && args[1]->IsInt32()
        && args[2]->IsInt32())
      {
        try
        {
          ::Windows::Foundation::Diagnostics::CausalityTraceLevel arg0 = static_cast<::Windows::Foundation::Diagnostics::CausalityTraceLevel>(args[0]->Int32Value());
          ::Windows::Foundation::Diagnostics::CausalitySource arg1 = static_cast<::Windows::Foundation::Diagnostics::CausalitySource>(args[1]->Int32Value());
          ::Windows::Foundation::Diagnostics::CausalitySynchronousWork arg2 = static_cast<::Windows::Foundation::Diagnostics::CausalitySynchronousWork>(args[2]->Int32Value());
          
          ::Windows::Foundation::Diagnostics::AsyncCausalityTracer::TraceSynchronousWorkCompletion(arg0, arg1, arg2);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }



    static v8::Handle<v8::Value> AddListener(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() < 2 || !args[0]->IsString() || !args[1]->IsFunction())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"wrong arguments, expected arguments are eventName(string),callback(function)")));
        return scope.Close(Undefined());
      }

      String::Value eventName(args[0]);
      auto str = *eventName;
      
      Local<Function> callback = args[1].As<Function>();
      
      ::Windows::Foundation::EventRegistrationToken registrationToken;
      if (NodeRT::Utils::CaseInsenstiveEquals(L"tracingStatusChanged", str))
      {
      
        try
        {
          std::shared_ptr<Persistent<Object>> callbackObjPtr(new Persistent<Object>(Persistent<Object>::New(NodeRT::Utils::CreateCallbackObjectInDomain(callback))), 
            [] (Persistent<Object> *ptr ) {
              NodeUtils::Async::RunOnMain([ptr]() {
                ptr->Dispose();
                delete ptr;
            });
          });

          registrationToken = ::Windows::Foundation::Diagnostics::AsyncCausalityTracer::TracingStatusChanged::add(
            ref new ::Windows::Foundation::EventHandler<::Windows::Foundation::Diagnostics::TracingStatusChangedEventArgs^>(
            [callbackObjPtr](::Platform::Object^ arg0, ::Windows::Foundation::Diagnostics::TracingStatusChangedEventArgs^ arg1) {
              NodeUtils::Async::RunOnMain([callbackObjPtr , arg0, arg1]() {
                TryCatch tryCatch;
              
                Handle<Value> error;

                Handle<Value> wrappedArg0 = CreateOpaqueWrapper(arg0);
                Handle<Value> wrappedArg1 = WrapTracingStatusChangedEventArgs(arg1);

                if (tryCatch.HasCaught())
                {
                  error = tryCatch.Exception()->ToObject();
                }
                else 
                {
                  error = Undefined();
                }


                if (wrappedArg0.IsEmpty()) wrappedArg0 = Undefined();
                if (wrappedArg1.IsEmpty()) wrappedArg1 = Undefined();

                Handle<Value> args[] = { wrappedArg0, wrappedArg1 };
                NodeRT::Utils::CallCallbackInDomain(*callbackObjPtr, _countof(args), args);
              });
            })
          );
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }

      }
      else 
      {
        ThrowException(Exception::Error(String::Concat(NodeRT::Utils::NewString(L"given event name isn't supported: "), args[0].As<String>())));
        return scope.Close(Undefined());
      }

      Local<Value> tokenMap = callback->GetHiddenValue(String::NewSymbol(REGISTRATION_TOKEN_MAP_PROPERTY_NAME));
                
      if (tokenMap.IsEmpty() || tokenMap->Equals(Undefined()))
      {
          tokenMap = Object::New();
          callback->SetHiddenValue(String::NewSymbol(REGISTRATION_TOKEN_MAP_PROPERTY_NAME), tokenMap);
      }

      tokenMap.As<Object>()->Set(args[1], CreateOpaqueWrapper(::Windows::Foundation::PropertyValue::CreateInt64(registrationToken.Value)));
                
      return scope.Close(Undefined());
    }

    static v8::Handle<v8::Value> RemoveListener(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() < 2 || !args[0]->IsString() || !args[1]->IsFunction())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"wrong arguments, expected a string and a callback")));
        return scope.Close(Undefined());
      }

      String::Value eventName(args[0]);
      auto str = *eventName;

      if ((NodeRT::Utils::CaseInsenstiveEquals(L"tracingStatusChanged", str)))
      {
        ThrowException(Exception::Error(String::Concat(NodeRT::Utils::NewString(L"given event name isn't supported: "), args[0].As<String>())));
        return scope.Close(Undefined());
      }

      Local<Function> callback = args[1].As<Function>();
      Handle<Value> tokenMap = callback->GetHiddenValue(String::NewSymbol(REGISTRATION_TOKEN_MAP_PROPERTY_NAME));
                
      if (tokenMap.IsEmpty() || tokenMap->Equals(Undefined()))
      {
        return scope.Close(Undefined());
      }

      Handle<Value> opaqueWrapperObj =  tokenMap.As<Object>()->Get(args[1]);

      if (opaqueWrapperObj.IsEmpty() || opaqueWrapperObj->Equals(Undefined()))
      {
        return scope.Close(Undefined());
      }

      OpaqueWrapper *opaqueWrapper = OpaqueWrapper::Unwrap<OpaqueWrapper>(opaqueWrapperObj.As<Object>());
            
      long long tokenValue = (long long) opaqueWrapper->GetObjectInstance();
      ::Windows::Foundation::EventRegistrationToken registrationToken;
      registrationToken.Value = tokenValue;
        
      try 
      {
        if (NodeRT::Utils::CaseInsenstiveEquals(L"tracingStatusChanged", str))
        {
          ::Windows::Foundation::Diagnostics::AsyncCausalityTracer::TracingStatusChanged::remove(registrationToken);
        }
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }

      tokenMap.As<Object>()->Delete(args[0].As<String>());

      return scope.Close(Undefined());
    }
  private:
    ::Windows::Foundation::Diagnostics::AsyncCausalityTracer^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapAsyncCausalityTracer(::Windows::Foundation::Diagnostics::AsyncCausalityTracer^ wintRtInstance);
    friend ::Windows::Foundation::Diagnostics::AsyncCausalityTracer^ UnwrapAsyncCausalityTracer(Handle<Value> value);
    friend bool IsAsyncCausalityTracerWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> AsyncCausalityTracer::s_constructorTemplate;

  v8::Handle<v8::Value> WrapAsyncCausalityTracer(::Windows::Foundation::Diagnostics::AsyncCausalityTracer^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(AsyncCausalityTracer::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Foundation::Diagnostics::AsyncCausalityTracer^ UnwrapAsyncCausalityTracer(Handle<Value> value)
  {
     return AsyncCausalityTracer::Unwrap<AsyncCausalityTracer>(value.As<Object>())->_instance;
  }

  void InitAsyncCausalityTracer(Handle<Object> exports)
  {
    AsyncCausalityTracer::Init(exports);
  }

  class IErrorReportingSettings : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("IErrorReportingSettings"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
            
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("setErrorOptions"), FunctionTemplate::New(SetErrorOptions)->GetFunction());
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("getErrorOptions"), FunctionTemplate::New(GetErrorOptions)->GetFunction());
      
                        
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("IErrorReportingSettings"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    IErrorReportingSettings(::Windows::Foundation::Diagnostics::IErrorReportingSettings^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Foundation::Diagnostics::IErrorReportingSettings^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::IErrorReportingSettings^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Foundation::Diagnostics::IErrorReportingSettings^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      IErrorReportingSettings *wrapperInstance = new IErrorReportingSettings(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  
    static Handle<Value> SetErrorOptions(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::IErrorReportingSettings^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      IErrorReportingSettings *wrapper = IErrorReportingSettings::Unwrap<IErrorReportingSettings>(args.This());

      if (args.Length() == 1
        && args[0]->IsInt32())
      {
        try
        {
          ::Windows::Foundation::Diagnostics::ErrorOptions arg0 = static_cast<::Windows::Foundation::Diagnostics::ErrorOptions>(args[0]->Int32Value());
          
          wrapper->_instance->SetErrorOptions(arg0);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> GetErrorOptions(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::IErrorReportingSettings^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      IErrorReportingSettings *wrapper = IErrorReportingSettings::Unwrap<IErrorReportingSettings>(args.This());

      if (args.Length() == 0)
      {
        try
        {
          ::Windows::Foundation::Diagnostics::ErrorOptions result;
          result = wrapper->_instance->GetErrorOptions();
          return scope.Close(Integer::New(static_cast<int>(result)));
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }





  private:
    ::Windows::Foundation::Diagnostics::IErrorReportingSettings^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapIErrorReportingSettings(::Windows::Foundation::Diagnostics::IErrorReportingSettings^ wintRtInstance);
    friend ::Windows::Foundation::Diagnostics::IErrorReportingSettings^ UnwrapIErrorReportingSettings(Handle<Value> value);
    friend bool IsIErrorReportingSettingsWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> IErrorReportingSettings::s_constructorTemplate;

  v8::Handle<v8::Value> WrapIErrorReportingSettings(::Windows::Foundation::Diagnostics::IErrorReportingSettings^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(IErrorReportingSettings::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Foundation::Diagnostics::IErrorReportingSettings^ UnwrapIErrorReportingSettings(Handle<Value> value)
  {
     return IErrorReportingSettings::Unwrap<IErrorReportingSettings>(value.As<Object>())->_instance;
  }

  void InitIErrorReportingSettings(Handle<Object> exports)
  {
    IErrorReportingSettings::Init(exports);
  }

  class RuntimeBrokerErrorSettings : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("RuntimeBrokerErrorSettings"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
            
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("setErrorOptions"), FunctionTemplate::New(SetErrorOptions)->GetFunction());
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("getErrorOptions"), FunctionTemplate::New(GetErrorOptions)->GetFunction());
      
                        
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("RuntimeBrokerErrorSettings"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    RuntimeBrokerErrorSettings(::Windows::Foundation::Diagnostics::RuntimeBrokerErrorSettings^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Foundation::Diagnostics::RuntimeBrokerErrorSettings^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::RuntimeBrokerErrorSettings^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Foundation::Diagnostics::RuntimeBrokerErrorSettings^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else if (args.Length() == 0)
      {
        try
        {
          winRtInstance = ref new ::Windows::Foundation::Diagnostics::RuntimeBrokerErrorSettings();
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      RuntimeBrokerErrorSettings *wrapperInstance = new RuntimeBrokerErrorSettings(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  
    static Handle<Value> SetErrorOptions(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::RuntimeBrokerErrorSettings^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      RuntimeBrokerErrorSettings *wrapper = RuntimeBrokerErrorSettings::Unwrap<RuntimeBrokerErrorSettings>(args.This());

      if (args.Length() == 1
        && args[0]->IsInt32())
      {
        try
        {
          ::Windows::Foundation::Diagnostics::ErrorOptions arg0 = static_cast<::Windows::Foundation::Diagnostics::ErrorOptions>(args[0]->Int32Value());
          
          wrapper->_instance->SetErrorOptions(arg0);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> GetErrorOptions(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::RuntimeBrokerErrorSettings^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      RuntimeBrokerErrorSettings *wrapper = RuntimeBrokerErrorSettings::Unwrap<RuntimeBrokerErrorSettings>(args.This());

      if (args.Length() == 0)
      {
        try
        {
          ::Windows::Foundation::Diagnostics::ErrorOptions result;
          result = wrapper->_instance->GetErrorOptions();
          return scope.Close(Integer::New(static_cast<int>(result)));
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }





  private:
    ::Windows::Foundation::Diagnostics::RuntimeBrokerErrorSettings^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapRuntimeBrokerErrorSettings(::Windows::Foundation::Diagnostics::RuntimeBrokerErrorSettings^ wintRtInstance);
    friend ::Windows::Foundation::Diagnostics::RuntimeBrokerErrorSettings^ UnwrapRuntimeBrokerErrorSettings(Handle<Value> value);
    friend bool IsRuntimeBrokerErrorSettingsWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> RuntimeBrokerErrorSettings::s_constructorTemplate;

  v8::Handle<v8::Value> WrapRuntimeBrokerErrorSettings(::Windows::Foundation::Diagnostics::RuntimeBrokerErrorSettings^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(RuntimeBrokerErrorSettings::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Foundation::Diagnostics::RuntimeBrokerErrorSettings^ UnwrapRuntimeBrokerErrorSettings(Handle<Value> value)
  {
     return RuntimeBrokerErrorSettings::Unwrap<RuntimeBrokerErrorSettings>(value.As<Object>())->_instance;
  }

  void InitRuntimeBrokerErrorSettings(Handle<Object> exports)
  {
    RuntimeBrokerErrorSettings::Init(exports);
  }

  class ILoggingChannel : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("ILoggingChannel"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
            
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("logMessage"), FunctionTemplate::New(LogMessage)->GetFunction());
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("logValuePair"), FunctionTemplate::New(LogValuePair)->GetFunction());
      
                  
      Local<Function> addListenerFunc = FunctionTemplate::New(AddListener)->GetFunction();
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("addListener"), addListenerFunc);
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("on"), addListenerFunc);
      Local<Function> removeListenerFunc = FunctionTemplate::New(RemoveListener)->GetFunction();
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("removeListener"), removeListenerFunc);
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("off"), removeListenerFunc);
            
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("enabled"), EnabledGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("level"), LevelGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("name"), NameGetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("ILoggingChannel"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    ILoggingChannel(::Windows::Foundation::Diagnostics::ILoggingChannel^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Foundation::Diagnostics::ILoggingChannel^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::ILoggingChannel^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Foundation::Diagnostics::ILoggingChannel^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      ILoggingChannel *wrapperInstance = new ILoggingChannel(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  
    static Handle<Value> LogMessage(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::ILoggingChannel^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      ILoggingChannel *wrapper = ILoggingChannel::Unwrap<ILoggingChannel>(args.This());

      if (args.Length() == 1
        && args[0]->IsString())
      {
        try
        {
          Platform::String^ arg0 = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(args[0])));
          
          wrapper->_instance->LogMessage(arg0);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else if (args.Length() == 2
        && args[0]->IsString()
        && args[1]->IsInt32())
      {
        try
        {
          Platform::String^ arg0 = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(args[0])));
          ::Windows::Foundation::Diagnostics::LoggingLevel arg1 = static_cast<::Windows::Foundation::Diagnostics::LoggingLevel>(args[1]->Int32Value());
          
          wrapper->_instance->LogMessage(arg0, arg1);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> LogValuePair(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::ILoggingChannel^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      ILoggingChannel *wrapper = ILoggingChannel::Unwrap<ILoggingChannel>(args.This());

      if (args.Length() == 2
        && args[0]->IsString()
        && args[1]->IsInt32())
      {
        try
        {
          Platform::String^ arg0 = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(args[0])));
          int arg1 = static_cast<int>(args[1]->Int32Value());
          
          wrapper->_instance->LogValuePair(arg0, arg1);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else if (args.Length() == 3
        && args[0]->IsString()
        && args[1]->IsInt32()
        && args[2]->IsInt32())
      {
        try
        {
          Platform::String^ arg0 = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(args[0])));
          int arg1 = static_cast<int>(args[1]->Int32Value());
          ::Windows::Foundation::Diagnostics::LoggingLevel arg2 = static_cast<::Windows::Foundation::Diagnostics::LoggingLevel>(args[2]->Int32Value());
          
          wrapper->_instance->LogValuePair(arg0, arg1, arg2);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }



    static Handle<Value> EnabledGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::ILoggingChannel^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ILoggingChannel *wrapper = ILoggingChannel::Unwrap<ILoggingChannel>(info.This());

      try 
      {
        bool result = wrapper->_instance->Enabled;
        return scope.Close(Boolean::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> LevelGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::ILoggingChannel^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ILoggingChannel *wrapper = ILoggingChannel::Unwrap<ILoggingChannel>(info.This());

      try 
      {
        ::Windows::Foundation::Diagnostics::LoggingLevel result = wrapper->_instance->Level;
        return scope.Close(Integer::New(static_cast<int>(result)));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> NameGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::ILoggingChannel^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ILoggingChannel *wrapper = ILoggingChannel::Unwrap<ILoggingChannel>(info.This());

      try 
      {
        Platform::String^ result = wrapper->_instance->Name;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    


    static v8::Handle<v8::Value> AddListener(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() < 2 || !args[0]->IsString() || !args[1]->IsFunction())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"wrong arguments, expected arguments are eventName(string),callback(function)")));
        return scope.Close(Undefined());
      }

      String::Value eventName(args[0]);
      auto str = *eventName;
      
      Local<Function> callback = args[1].As<Function>();
      
      ::Windows::Foundation::EventRegistrationToken registrationToken;
      if (NodeRT::Utils::CaseInsenstiveEquals(L"loggingEnabled", str))
      {
        if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::ILoggingChannel^>(args.This()))
        {
          ThrowException(Exception::Error(NodeRT::Utils::NewString(L"The caller of this method isn't of the expected type or internal WinRt object was disposed")));
          return scope.Close(Undefined());
        }
        ILoggingChannel *wrapper = ILoggingChannel::Unwrap<ILoggingChannel>(args.This());
      
        try
        {
          std::shared_ptr<Persistent<Object>> callbackObjPtr(new Persistent<Object>(Persistent<Object>::New(NodeRT::Utils::CreateCallbackObjectInDomain(callback))), 
            [] (Persistent<Object> *ptr ) {
              NodeUtils::Async::RunOnMain([ptr]() {
                ptr->Dispose();
                delete ptr;
            });
          });

          registrationToken = wrapper->_instance->LoggingEnabled::add(
            ref new ::Windows::Foundation::TypedEventHandler<::Windows::Foundation::Diagnostics::ILoggingChannel^, ::Platform::Object^>(
            [callbackObjPtr](::Windows::Foundation::Diagnostics::ILoggingChannel^ arg0, ::Platform::Object^ arg1) {
              NodeUtils::Async::RunOnMain([callbackObjPtr , arg0, arg1]() {
                TryCatch tryCatch;
              
                Handle<Value> error;

                Handle<Value> wrappedArg0 = WrapILoggingChannel(arg0);
                Handle<Value> wrappedArg1 = CreateOpaqueWrapper(arg1);

                if (tryCatch.HasCaught())
                {
                  error = tryCatch.Exception()->ToObject();
                }
                else 
                {
                  error = Undefined();
                }


                if (wrappedArg0.IsEmpty()) wrappedArg0 = Undefined();
                if (wrappedArg1.IsEmpty()) wrappedArg1 = Undefined();

                Handle<Value> args[] = { wrappedArg0, wrappedArg1 };
                NodeRT::Utils::CallCallbackInDomain(*callbackObjPtr, _countof(args), args);
              });
            })
          );
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }

      }
      else 
      {
        ThrowException(Exception::Error(String::Concat(NodeRT::Utils::NewString(L"given event name isn't supported: "), args[0].As<String>())));
        return scope.Close(Undefined());
      }

      Local<Value> tokenMap = callback->GetHiddenValue(String::NewSymbol(REGISTRATION_TOKEN_MAP_PROPERTY_NAME));
                
      if (tokenMap.IsEmpty() || tokenMap->Equals(Undefined()))
      {
          tokenMap = Object::New();
          callback->SetHiddenValue(String::NewSymbol(REGISTRATION_TOKEN_MAP_PROPERTY_NAME), tokenMap);
      }

      tokenMap.As<Object>()->Set(args[1], CreateOpaqueWrapper(::Windows::Foundation::PropertyValue::CreateInt64(registrationToken.Value)));
                
      return scope.Close(Undefined());
    }

    static v8::Handle<v8::Value> RemoveListener(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() < 2 || !args[0]->IsString() || !args[1]->IsFunction())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"wrong arguments, expected a string and a callback")));
        return scope.Close(Undefined());
      }

      String::Value eventName(args[0]);
      auto str = *eventName;

      if ((NodeRT::Utils::CaseInsenstiveEquals(L"loggingEnabled", str)))
      {
        ThrowException(Exception::Error(String::Concat(NodeRT::Utils::NewString(L"given event name isn't supported: "), args[0].As<String>())));
        return scope.Close(Undefined());
      }

      Local<Function> callback = args[1].As<Function>();
      Handle<Value> tokenMap = callback->GetHiddenValue(String::NewSymbol(REGISTRATION_TOKEN_MAP_PROPERTY_NAME));
                
      if (tokenMap.IsEmpty() || tokenMap->Equals(Undefined()))
      {
        return scope.Close(Undefined());
      }

      Handle<Value> opaqueWrapperObj =  tokenMap.As<Object>()->Get(args[1]);

      if (opaqueWrapperObj.IsEmpty() || opaqueWrapperObj->Equals(Undefined()))
      {
        return scope.Close(Undefined());
      }

      OpaqueWrapper *opaqueWrapper = OpaqueWrapper::Unwrap<OpaqueWrapper>(opaqueWrapperObj.As<Object>());
            
      long long tokenValue = (long long) opaqueWrapper->GetObjectInstance();
      ::Windows::Foundation::EventRegistrationToken registrationToken;
      registrationToken.Value = tokenValue;
        
      try 
      {
        if (NodeRT::Utils::CaseInsenstiveEquals(L"loggingEnabled", str))
        {
          if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::ILoggingChannel^>(args.This()))
          {
            ThrowException(Exception::Error(NodeRT::Utils::NewString(L"The caller of this method isn't of the expected type or internal WinRt object was disposed")));
            return scope.Close(Undefined());
          }
          ILoggingChannel *wrapper = ILoggingChannel::Unwrap<ILoggingChannel>(args.This());
          wrapper->_instance->LoggingEnabled::remove(registrationToken);
        }
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }

      tokenMap.As<Object>()->Delete(args[0].As<String>());

      return scope.Close(Undefined());
    }
  private:
    ::Windows::Foundation::Diagnostics::ILoggingChannel^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapILoggingChannel(::Windows::Foundation::Diagnostics::ILoggingChannel^ wintRtInstance);
    friend ::Windows::Foundation::Diagnostics::ILoggingChannel^ UnwrapILoggingChannel(Handle<Value> value);
    friend bool IsILoggingChannelWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> ILoggingChannel::s_constructorTemplate;

  v8::Handle<v8::Value> WrapILoggingChannel(::Windows::Foundation::Diagnostics::ILoggingChannel^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(ILoggingChannel::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Foundation::Diagnostics::ILoggingChannel^ UnwrapILoggingChannel(Handle<Value> value)
  {
     return ILoggingChannel::Unwrap<ILoggingChannel>(value.As<Object>())->_instance;
  }

  void InitILoggingChannel(Handle<Object> exports)
  {
    ILoggingChannel::Init(exports);
  }

  class LoggingChannel : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("LoggingChannel"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
            
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("logMessage"), FunctionTemplate::New(LogMessage)->GetFunction());
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("logValuePair"), FunctionTemplate::New(LogValuePair)->GetFunction());
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("close"), FunctionTemplate::New(Close)->GetFunction());
      
                  
      Local<Function> addListenerFunc = FunctionTemplate::New(AddListener)->GetFunction();
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("addListener"), addListenerFunc);
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("on"), addListenerFunc);
      Local<Function> removeListenerFunc = FunctionTemplate::New(RemoveListener)->GetFunction();
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("removeListener"), removeListenerFunc);
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("off"), removeListenerFunc);
            
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("enabled"), EnabledGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("level"), LevelGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("name"), NameGetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("LoggingChannel"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    LoggingChannel(::Windows::Foundation::Diagnostics::LoggingChannel^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Foundation::Diagnostics::LoggingChannel^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::LoggingChannel^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Foundation::Diagnostics::LoggingChannel^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else if (args.Length() == 1
        && args[0]->IsString())
      {
        try
        {
          Platform::String^ arg0 = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(args[0])));
          
          winRtInstance = ref new ::Windows::Foundation::Diagnostics::LoggingChannel(arg0);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      LoggingChannel *wrapperInstance = new LoggingChannel(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  
    static Handle<Value> LogMessage(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::LoggingChannel^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      LoggingChannel *wrapper = LoggingChannel::Unwrap<LoggingChannel>(args.This());

      if (args.Length() == 1
        && args[0]->IsString())
      {
        try
        {
          Platform::String^ arg0 = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(args[0])));
          
          wrapper->_instance->LogMessage(arg0);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else if (args.Length() == 2
        && args[0]->IsString()
        && args[1]->IsInt32())
      {
        try
        {
          Platform::String^ arg0 = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(args[0])));
          ::Windows::Foundation::Diagnostics::LoggingLevel arg1 = static_cast<::Windows::Foundation::Diagnostics::LoggingLevel>(args[1]->Int32Value());
          
          wrapper->_instance->LogMessage(arg0, arg1);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> LogValuePair(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::LoggingChannel^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      LoggingChannel *wrapper = LoggingChannel::Unwrap<LoggingChannel>(args.This());

      if (args.Length() == 2
        && args[0]->IsString()
        && args[1]->IsInt32())
      {
        try
        {
          Platform::String^ arg0 = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(args[0])));
          int arg1 = static_cast<int>(args[1]->Int32Value());
          
          wrapper->_instance->LogValuePair(arg0, arg1);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else if (args.Length() == 3
        && args[0]->IsString()
        && args[1]->IsInt32()
        && args[2]->IsInt32())
      {
        try
        {
          Platform::String^ arg0 = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(args[0])));
          int arg1 = static_cast<int>(args[1]->Int32Value());
          ::Windows::Foundation::Diagnostics::LoggingLevel arg2 = static_cast<::Windows::Foundation::Diagnostics::LoggingLevel>(args[2]->Int32Value());
          
          wrapper->_instance->LogValuePair(arg0, arg1, arg2);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> Close(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::LoggingChannel^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      LoggingChannel *wrapper = LoggingChannel::Unwrap<LoggingChannel>(args.This());

      if (args.Length() == 0)
      {
        try
        {
          delete wrapper->_instance;
          wrapper->_instance = nullptr;
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }




    static Handle<Value> EnabledGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::LoggingChannel^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      LoggingChannel *wrapper = LoggingChannel::Unwrap<LoggingChannel>(info.This());

      try 
      {
        bool result = wrapper->_instance->Enabled;
        return scope.Close(Boolean::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> LevelGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::LoggingChannel^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      LoggingChannel *wrapper = LoggingChannel::Unwrap<LoggingChannel>(info.This());

      try 
      {
        ::Windows::Foundation::Diagnostics::LoggingLevel result = wrapper->_instance->Level;
        return scope.Close(Integer::New(static_cast<int>(result)));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> NameGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::LoggingChannel^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      LoggingChannel *wrapper = LoggingChannel::Unwrap<LoggingChannel>(info.This());

      try 
      {
        Platform::String^ result = wrapper->_instance->Name;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    


    static v8::Handle<v8::Value> AddListener(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() < 2 || !args[0]->IsString() || !args[1]->IsFunction())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"wrong arguments, expected arguments are eventName(string),callback(function)")));
        return scope.Close(Undefined());
      }

      String::Value eventName(args[0]);
      auto str = *eventName;
      
      Local<Function> callback = args[1].As<Function>();
      
      ::Windows::Foundation::EventRegistrationToken registrationToken;
      if (NodeRT::Utils::CaseInsenstiveEquals(L"loggingEnabled", str))
      {
        if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::LoggingChannel^>(args.This()))
        {
          ThrowException(Exception::Error(NodeRT::Utils::NewString(L"The caller of this method isn't of the expected type or internal WinRt object was disposed")));
          return scope.Close(Undefined());
        }
        LoggingChannel *wrapper = LoggingChannel::Unwrap<LoggingChannel>(args.This());
      
        try
        {
          std::shared_ptr<Persistent<Object>> callbackObjPtr(new Persistent<Object>(Persistent<Object>::New(NodeRT::Utils::CreateCallbackObjectInDomain(callback))), 
            [] (Persistent<Object> *ptr ) {
              NodeUtils::Async::RunOnMain([ptr]() {
                ptr->Dispose();
                delete ptr;
            });
          });

          registrationToken = wrapper->_instance->LoggingEnabled::add(
            ref new ::Windows::Foundation::TypedEventHandler<::Windows::Foundation::Diagnostics::ILoggingChannel^, ::Platform::Object^>(
            [callbackObjPtr](::Windows::Foundation::Diagnostics::ILoggingChannel^ arg0, ::Platform::Object^ arg1) {
              NodeUtils::Async::RunOnMain([callbackObjPtr , arg0, arg1]() {
                TryCatch tryCatch;
              
                Handle<Value> error;

                Handle<Value> wrappedArg0 = WrapILoggingChannel(arg0);
                Handle<Value> wrappedArg1 = CreateOpaqueWrapper(arg1);

                if (tryCatch.HasCaught())
                {
                  error = tryCatch.Exception()->ToObject();
                }
                else 
                {
                  error = Undefined();
                }


                if (wrappedArg0.IsEmpty()) wrappedArg0 = Undefined();
                if (wrappedArg1.IsEmpty()) wrappedArg1 = Undefined();

                Handle<Value> args[] = { wrappedArg0, wrappedArg1 };
                NodeRT::Utils::CallCallbackInDomain(*callbackObjPtr, _countof(args), args);
              });
            })
          );
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }

      }
      else 
      {
        ThrowException(Exception::Error(String::Concat(NodeRT::Utils::NewString(L"given event name isn't supported: "), args[0].As<String>())));
        return scope.Close(Undefined());
      }

      Local<Value> tokenMap = callback->GetHiddenValue(String::NewSymbol(REGISTRATION_TOKEN_MAP_PROPERTY_NAME));
                
      if (tokenMap.IsEmpty() || tokenMap->Equals(Undefined()))
      {
          tokenMap = Object::New();
          callback->SetHiddenValue(String::NewSymbol(REGISTRATION_TOKEN_MAP_PROPERTY_NAME), tokenMap);
      }

      tokenMap.As<Object>()->Set(args[1], CreateOpaqueWrapper(::Windows::Foundation::PropertyValue::CreateInt64(registrationToken.Value)));
                
      return scope.Close(Undefined());
    }

    static v8::Handle<v8::Value> RemoveListener(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() < 2 || !args[0]->IsString() || !args[1]->IsFunction())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"wrong arguments, expected a string and a callback")));
        return scope.Close(Undefined());
      }

      String::Value eventName(args[0]);
      auto str = *eventName;

      if ((NodeRT::Utils::CaseInsenstiveEquals(L"loggingEnabled", str)))
      {
        ThrowException(Exception::Error(String::Concat(NodeRT::Utils::NewString(L"given event name isn't supported: "), args[0].As<String>())));
        return scope.Close(Undefined());
      }

      Local<Function> callback = args[1].As<Function>();
      Handle<Value> tokenMap = callback->GetHiddenValue(String::NewSymbol(REGISTRATION_TOKEN_MAP_PROPERTY_NAME));
                
      if (tokenMap.IsEmpty() || tokenMap->Equals(Undefined()))
      {
        return scope.Close(Undefined());
      }

      Handle<Value> opaqueWrapperObj =  tokenMap.As<Object>()->Get(args[1]);

      if (opaqueWrapperObj.IsEmpty() || opaqueWrapperObj->Equals(Undefined()))
      {
        return scope.Close(Undefined());
      }

      OpaqueWrapper *opaqueWrapper = OpaqueWrapper::Unwrap<OpaqueWrapper>(opaqueWrapperObj.As<Object>());
            
      long long tokenValue = (long long) opaqueWrapper->GetObjectInstance();
      ::Windows::Foundation::EventRegistrationToken registrationToken;
      registrationToken.Value = tokenValue;
        
      try 
      {
        if (NodeRT::Utils::CaseInsenstiveEquals(L"loggingEnabled", str))
        {
          if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::LoggingChannel^>(args.This()))
          {
            ThrowException(Exception::Error(NodeRT::Utils::NewString(L"The caller of this method isn't of the expected type or internal WinRt object was disposed")));
            return scope.Close(Undefined());
          }
          LoggingChannel *wrapper = LoggingChannel::Unwrap<LoggingChannel>(args.This());
          wrapper->_instance->LoggingEnabled::remove(registrationToken);
        }
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }

      tokenMap.As<Object>()->Delete(args[0].As<String>());

      return scope.Close(Undefined());
    }
  private:
    ::Windows::Foundation::Diagnostics::LoggingChannel^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapLoggingChannel(::Windows::Foundation::Diagnostics::LoggingChannel^ wintRtInstance);
    friend ::Windows::Foundation::Diagnostics::LoggingChannel^ UnwrapLoggingChannel(Handle<Value> value);
    friend bool IsLoggingChannelWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> LoggingChannel::s_constructorTemplate;

  v8::Handle<v8::Value> WrapLoggingChannel(::Windows::Foundation::Diagnostics::LoggingChannel^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(LoggingChannel::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Foundation::Diagnostics::LoggingChannel^ UnwrapLoggingChannel(Handle<Value> value)
  {
     return LoggingChannel::Unwrap<LoggingChannel>(value.As<Object>())->_instance;
  }

  void InitLoggingChannel(Handle<Object> exports)
  {
    LoggingChannel::Init(exports);
  }

  class LoggingActivity : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("LoggingActivity"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
            
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("close"), FunctionTemplate::New(Close)->GetFunction());
      
                        
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("id"), IdGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("name"), NameGetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("LoggingActivity"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    LoggingActivity(::Windows::Foundation::Diagnostics::LoggingActivity^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Foundation::Diagnostics::LoggingActivity^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::LoggingActivity^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Foundation::Diagnostics::LoggingActivity^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else if (args.Length() == 2
        && args[0]->IsString()
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::ILoggingChannel^>(args[1]))
      {
        try
        {
          Platform::String^ arg0 = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(args[0])));
          ::Windows::Foundation::Diagnostics::ILoggingChannel^ arg1 = UnwrapILoggingChannel(args[1]);
          
          winRtInstance = ref new ::Windows::Foundation::Diagnostics::LoggingActivity(arg0,arg1);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else if (args.Length() == 3
        && args[0]->IsString()
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::ILoggingChannel^>(args[1])
        && args[2]->IsInt32())
      {
        try
        {
          Platform::String^ arg0 = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(args[0])));
          ::Windows::Foundation::Diagnostics::ILoggingChannel^ arg1 = UnwrapILoggingChannel(args[1]);
          ::Windows::Foundation::Diagnostics::LoggingLevel arg2 = static_cast<::Windows::Foundation::Diagnostics::LoggingLevel>(args[2]->Int32Value());
          
          winRtInstance = ref new ::Windows::Foundation::Diagnostics::LoggingActivity(arg0,arg1,arg2);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      LoggingActivity *wrapperInstance = new LoggingActivity(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  
    static Handle<Value> Close(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::LoggingActivity^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      LoggingActivity *wrapper = LoggingActivity::Unwrap<LoggingActivity>(args.This());

      if (args.Length() == 0)
      {
        try
        {
          delete wrapper->_instance;
          wrapper->_instance = nullptr;
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }




    static Handle<Value> IdGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::LoggingActivity^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      LoggingActivity *wrapper = LoggingActivity::Unwrap<LoggingActivity>(info.This());

      try 
      {
        ::Platform::Guid result = wrapper->_instance->Id;
        return scope.Close(NodeRT::Utils::GuidToJs(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> NameGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::LoggingActivity^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      LoggingActivity *wrapper = LoggingActivity::Unwrap<LoggingActivity>(info.This());

      try 
      {
        Platform::String^ result = wrapper->_instance->Name;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    


  private:
    ::Windows::Foundation::Diagnostics::LoggingActivity^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapLoggingActivity(::Windows::Foundation::Diagnostics::LoggingActivity^ wintRtInstance);
    friend ::Windows::Foundation::Diagnostics::LoggingActivity^ UnwrapLoggingActivity(Handle<Value> value);
    friend bool IsLoggingActivityWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> LoggingActivity::s_constructorTemplate;

  v8::Handle<v8::Value> WrapLoggingActivity(::Windows::Foundation::Diagnostics::LoggingActivity^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(LoggingActivity::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Foundation::Diagnostics::LoggingActivity^ UnwrapLoggingActivity(Handle<Value> value)
  {
     return LoggingActivity::Unwrap<LoggingActivity>(value.As<Object>())->_instance;
  }

  void InitLoggingActivity(Handle<Object> exports)
  {
    LoggingActivity::Init(exports);
  }

  class ILoggingSession : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("ILoggingSession"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
      Handle<Value> asyncSymbol = String::NewSymbol("__winRtAsync__");
      Handle<Function> func;
            
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("addLoggingChannel"), FunctionTemplate::New(AddLoggingChannel)->GetFunction());
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("removeLoggingChannel"), FunctionTemplate::New(RemoveLoggingChannel)->GetFunction());
      
            
      func = FunctionTemplate::New(SaveToFileAsync)->GetFunction();
      func->Set(asyncSymbol, True(), PropertyAttribute::DontEnum);
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("saveToFileAsync"), func);
      
                  
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("name"), NameGetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("ILoggingSession"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    ILoggingSession(::Windows::Foundation::Diagnostics::ILoggingSession^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Foundation::Diagnostics::ILoggingSession^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::ILoggingSession^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Foundation::Diagnostics::ILoggingSession^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      ILoggingSession *wrapperInstance = new ILoggingSession(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


    static Handle<Value> SaveToFileAsync(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::ILoggingSession^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      if (args.Length() == 0 || !args[args.Length() -1]->IsFunction())
      {
          ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: No callback was given")));
          return scope.Close(Undefined());
      }

      ILoggingSession *wrapper = ILoggingSession::Unwrap<ILoggingSession>(args.This());

      ::Windows::Foundation::IAsyncOperation<::Windows::Storage::StorageFile^>^ op;
    

      if (args.Length() == 3
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::Storage::IStorageFolder^>(args[0])
        && args[1]->IsString())
      {
        try
        {
          ::Windows::Storage::IStorageFolder^ arg0 = dynamic_cast<::Windows::Storage::IStorageFolder^>(NodeRT::Utils::GetObjectInstance(args[0]));
          Platform::String^ arg1 = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(args[1])));
          
          op = wrapper->_instance->SaveToFileAsync(arg0,arg1);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }
    
      auto opTask = create_task(op);
      uv_async_t* asyncToken = NodeUtils::Async::GetAsyncToken(args[args.Length() -1].As<Function>());

      opTask.then( [asyncToken] (task<::Windows::Storage::StorageFile^> t) 
      {	
        try
        {
          auto result = t.get();
          NodeUtils::Async::RunCallbackOnMain(asyncToken, [result](NodeUtils::InvokeCallbackDelegate invokeCallback) {


            TryCatch tryCatch;
            Handle<Value> error; 
            Handle<Value> arg1 = NodeRT::Utils::CreateExternalWinRTObject("Windows.Storage", "StorageFile", result);
            if (tryCatch.HasCaught())
            {
              error = tryCatch.Exception()->ToObject();
            }
            else 
            {
              error = Undefined();
            }
            if (arg1.IsEmpty()) arg1 = Undefined();
            Handle<Value> args[] = {error, arg1};

            invokeCallback(_countof(args), args);
          });
        }
        catch (Platform::Exception^ exception)
        {
          NodeUtils::Async::RunCallbackOnMain(asyncToken, [exception](NodeUtils::InvokeCallbackDelegate invokeCallback) {
             
            Handle<Value> error = NodeRT::Utils::WinRtExceptionToJsError(exception);
        
            Handle<Value> args[] = {error};
            invokeCallback(_countof(args), args);
          });
        }  		
      });

      return scope.Close(Undefined());
    }
  
    static Handle<Value> AddLoggingChannel(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::ILoggingSession^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      ILoggingSession *wrapper = ILoggingSession::Unwrap<ILoggingSession>(args.This());

      if (args.Length() == 1
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::ILoggingChannel^>(args[0]))
      {
        try
        {
          ::Windows::Foundation::Diagnostics::ILoggingChannel^ arg0 = UnwrapILoggingChannel(args[0]);
          
          wrapper->_instance->AddLoggingChannel(arg0);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else if (args.Length() == 2
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::ILoggingChannel^>(args[0])
        && args[1]->IsInt32())
      {
        try
        {
          ::Windows::Foundation::Diagnostics::ILoggingChannel^ arg0 = UnwrapILoggingChannel(args[0]);
          ::Windows::Foundation::Diagnostics::LoggingLevel arg1 = static_cast<::Windows::Foundation::Diagnostics::LoggingLevel>(args[1]->Int32Value());
          
          wrapper->_instance->AddLoggingChannel(arg0, arg1);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> RemoveLoggingChannel(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::ILoggingSession^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      ILoggingSession *wrapper = ILoggingSession::Unwrap<ILoggingSession>(args.This());

      if (args.Length() == 1
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::ILoggingChannel^>(args[0]))
      {
        try
        {
          ::Windows::Foundation::Diagnostics::ILoggingChannel^ arg0 = UnwrapILoggingChannel(args[0]);
          
          wrapper->_instance->RemoveLoggingChannel(arg0);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }



    static Handle<Value> NameGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::ILoggingSession^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ILoggingSession *wrapper = ILoggingSession::Unwrap<ILoggingSession>(info.This());

      try 
      {
        Platform::String^ result = wrapper->_instance->Name;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    


  private:
    ::Windows::Foundation::Diagnostics::ILoggingSession^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapILoggingSession(::Windows::Foundation::Diagnostics::ILoggingSession^ wintRtInstance);
    friend ::Windows::Foundation::Diagnostics::ILoggingSession^ UnwrapILoggingSession(Handle<Value> value);
    friend bool IsILoggingSessionWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> ILoggingSession::s_constructorTemplate;

  v8::Handle<v8::Value> WrapILoggingSession(::Windows::Foundation::Diagnostics::ILoggingSession^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(ILoggingSession::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Foundation::Diagnostics::ILoggingSession^ UnwrapILoggingSession(Handle<Value> value)
  {
     return ILoggingSession::Unwrap<ILoggingSession>(value.As<Object>())->_instance;
  }

  void InitILoggingSession(Handle<Object> exports)
  {
    ILoggingSession::Init(exports);
  }

  class LoggingSession : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("LoggingSession"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
      Handle<Value> asyncSymbol = String::NewSymbol("__winRtAsync__");
      Handle<Function> func;
            
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("addLoggingChannel"), FunctionTemplate::New(AddLoggingChannel)->GetFunction());
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("removeLoggingChannel"), FunctionTemplate::New(RemoveLoggingChannel)->GetFunction());
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("close"), FunctionTemplate::New(Close)->GetFunction());
      
            
      func = FunctionTemplate::New(SaveToFileAsync)->GetFunction();
      func->Set(asyncSymbol, True(), PropertyAttribute::DontEnum);
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("saveToFileAsync"), func);
      
                  
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("name"), NameGetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("LoggingSession"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    LoggingSession(::Windows::Foundation::Diagnostics::LoggingSession^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Foundation::Diagnostics::LoggingSession^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::LoggingSession^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Foundation::Diagnostics::LoggingSession^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else if (args.Length() == 1
        && args[0]->IsString())
      {
        try
        {
          Platform::String^ arg0 = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(args[0])));
          
          winRtInstance = ref new ::Windows::Foundation::Diagnostics::LoggingSession(arg0);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      LoggingSession *wrapperInstance = new LoggingSession(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


    static Handle<Value> SaveToFileAsync(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::LoggingSession^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      if (args.Length() == 0 || !args[args.Length() -1]->IsFunction())
      {
          ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: No callback was given")));
          return scope.Close(Undefined());
      }

      LoggingSession *wrapper = LoggingSession::Unwrap<LoggingSession>(args.This());

      ::Windows::Foundation::IAsyncOperation<::Windows::Storage::StorageFile^>^ op;
    

      if (args.Length() == 3
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::Storage::IStorageFolder^>(args[0])
        && args[1]->IsString())
      {
        try
        {
          ::Windows::Storage::IStorageFolder^ arg0 = dynamic_cast<::Windows::Storage::IStorageFolder^>(NodeRT::Utils::GetObjectInstance(args[0]));
          Platform::String^ arg1 = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(args[1])));
          
          op = wrapper->_instance->SaveToFileAsync(arg0,arg1);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }
    
      auto opTask = create_task(op);
      uv_async_t* asyncToken = NodeUtils::Async::GetAsyncToken(args[args.Length() -1].As<Function>());

      opTask.then( [asyncToken] (task<::Windows::Storage::StorageFile^> t) 
      {	
        try
        {
          auto result = t.get();
          NodeUtils::Async::RunCallbackOnMain(asyncToken, [result](NodeUtils::InvokeCallbackDelegate invokeCallback) {


            TryCatch tryCatch;
            Handle<Value> error; 
            Handle<Value> arg1 = NodeRT::Utils::CreateExternalWinRTObject("Windows.Storage", "StorageFile", result);
            if (tryCatch.HasCaught())
            {
              error = tryCatch.Exception()->ToObject();
            }
            else 
            {
              error = Undefined();
            }
            if (arg1.IsEmpty()) arg1 = Undefined();
            Handle<Value> args[] = {error, arg1};

            invokeCallback(_countof(args), args);
          });
        }
        catch (Platform::Exception^ exception)
        {
          NodeUtils::Async::RunCallbackOnMain(asyncToken, [exception](NodeUtils::InvokeCallbackDelegate invokeCallback) {
             
            Handle<Value> error = NodeRT::Utils::WinRtExceptionToJsError(exception);
        
            Handle<Value> args[] = {error};
            invokeCallback(_countof(args), args);
          });
        }  		
      });

      return scope.Close(Undefined());
    }
  
    static Handle<Value> AddLoggingChannel(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::LoggingSession^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      LoggingSession *wrapper = LoggingSession::Unwrap<LoggingSession>(args.This());

      if (args.Length() == 1
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::ILoggingChannel^>(args[0]))
      {
        try
        {
          ::Windows::Foundation::Diagnostics::ILoggingChannel^ arg0 = UnwrapILoggingChannel(args[0]);
          
          wrapper->_instance->AddLoggingChannel(arg0);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else if (args.Length() == 2
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::ILoggingChannel^>(args[0])
        && args[1]->IsInt32())
      {
        try
        {
          ::Windows::Foundation::Diagnostics::ILoggingChannel^ arg0 = UnwrapILoggingChannel(args[0]);
          ::Windows::Foundation::Diagnostics::LoggingLevel arg1 = static_cast<::Windows::Foundation::Diagnostics::LoggingLevel>(args[1]->Int32Value());
          
          wrapper->_instance->AddLoggingChannel(arg0, arg1);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> RemoveLoggingChannel(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::LoggingSession^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      LoggingSession *wrapper = LoggingSession::Unwrap<LoggingSession>(args.This());

      if (args.Length() == 1
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::ILoggingChannel^>(args[0]))
      {
        try
        {
          ::Windows::Foundation::Diagnostics::ILoggingChannel^ arg0 = UnwrapILoggingChannel(args[0]);
          
          wrapper->_instance->RemoveLoggingChannel(arg0);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> Close(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::LoggingSession^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      LoggingSession *wrapper = LoggingSession::Unwrap<LoggingSession>(args.This());

      if (args.Length() == 0)
      {
        try
        {
          delete wrapper->_instance;
          wrapper->_instance = nullptr;
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }




    static Handle<Value> NameGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::LoggingSession^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      LoggingSession *wrapper = LoggingSession::Unwrap<LoggingSession>(info.This());

      try 
      {
        Platform::String^ result = wrapper->_instance->Name;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    


  private:
    ::Windows::Foundation::Diagnostics::LoggingSession^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapLoggingSession(::Windows::Foundation::Diagnostics::LoggingSession^ wintRtInstance);
    friend ::Windows::Foundation::Diagnostics::LoggingSession^ UnwrapLoggingSession(Handle<Value> value);
    friend bool IsLoggingSessionWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> LoggingSession::s_constructorTemplate;

  v8::Handle<v8::Value> WrapLoggingSession(::Windows::Foundation::Diagnostics::LoggingSession^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(LoggingSession::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Foundation::Diagnostics::LoggingSession^ UnwrapLoggingSession(Handle<Value> value)
  {
     return LoggingSession::Unwrap<LoggingSession>(value.As<Object>())->_instance;
  }

  void InitLoggingSession(Handle<Object> exports)
  {
    LoggingSession::Init(exports);
  }

  class LogFileGeneratedEventArgs : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("LogFileGeneratedEventArgs"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
                              
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("file"), FileGetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("LogFileGeneratedEventArgs"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    LogFileGeneratedEventArgs(::Windows::Foundation::Diagnostics::LogFileGeneratedEventArgs^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Foundation::Diagnostics::LogFileGeneratedEventArgs^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::LogFileGeneratedEventArgs^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Foundation::Diagnostics::LogFileGeneratedEventArgs^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      LogFileGeneratedEventArgs *wrapperInstance = new LogFileGeneratedEventArgs(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  



    static Handle<Value> FileGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::LogFileGeneratedEventArgs^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      LogFileGeneratedEventArgs *wrapper = LogFileGeneratedEventArgs::Unwrap<LogFileGeneratedEventArgs>(info.This());

      try 
      {
        ::Windows::Storage::StorageFile^ result = wrapper->_instance->File;
        return scope.Close(NodeRT::Utils::CreateExternalWinRTObject("Windows.Storage", "StorageFile", result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    


  private:
    ::Windows::Foundation::Diagnostics::LogFileGeneratedEventArgs^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapLogFileGeneratedEventArgs(::Windows::Foundation::Diagnostics::LogFileGeneratedEventArgs^ wintRtInstance);
    friend ::Windows::Foundation::Diagnostics::LogFileGeneratedEventArgs^ UnwrapLogFileGeneratedEventArgs(Handle<Value> value);
    friend bool IsLogFileGeneratedEventArgsWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> LogFileGeneratedEventArgs::s_constructorTemplate;

  v8::Handle<v8::Value> WrapLogFileGeneratedEventArgs(::Windows::Foundation::Diagnostics::LogFileGeneratedEventArgs^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(LogFileGeneratedEventArgs::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Foundation::Diagnostics::LogFileGeneratedEventArgs^ UnwrapLogFileGeneratedEventArgs(Handle<Value> value)
  {
     return LogFileGeneratedEventArgs::Unwrap<LogFileGeneratedEventArgs>(value.As<Object>())->_instance;
  }

  void InitLogFileGeneratedEventArgs(Handle<Object> exports)
  {
    LogFileGeneratedEventArgs::Init(exports);
  }

  class IFileLoggingSession : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("IFileLoggingSession"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
      Handle<Value> asyncSymbol = String::NewSymbol("__winRtAsync__");
      Handle<Function> func;
            
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("addLoggingChannel"), FunctionTemplate::New(AddLoggingChannel)->GetFunction());
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("removeLoggingChannel"), FunctionTemplate::New(RemoveLoggingChannel)->GetFunction());
      
            
      func = FunctionTemplate::New(CloseAndSaveToFileAsync)->GetFunction();
      func->Set(asyncSymbol, True(), PropertyAttribute::DontEnum);
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("closeAndSaveToFileAsync"), func);
      
            
      Local<Function> addListenerFunc = FunctionTemplate::New(AddListener)->GetFunction();
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("addListener"), addListenerFunc);
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("on"), addListenerFunc);
      Local<Function> removeListenerFunc = FunctionTemplate::New(RemoveListener)->GetFunction();
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("removeListener"), removeListenerFunc);
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("off"), removeListenerFunc);
            
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("name"), NameGetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("IFileLoggingSession"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    IFileLoggingSession(::Windows::Foundation::Diagnostics::IFileLoggingSession^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Foundation::Diagnostics::IFileLoggingSession^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::IFileLoggingSession^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Foundation::Diagnostics::IFileLoggingSession^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      IFileLoggingSession *wrapperInstance = new IFileLoggingSession(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


    static Handle<Value> CloseAndSaveToFileAsync(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::IFileLoggingSession^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      if (args.Length() == 0 || !args[args.Length() -1]->IsFunction())
      {
          ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: No callback was given")));
          return scope.Close(Undefined());
      }

      IFileLoggingSession *wrapper = IFileLoggingSession::Unwrap<IFileLoggingSession>(args.This());

      ::Windows::Foundation::IAsyncOperation<::Windows::Storage::StorageFile^>^ op;
    

      if (args.Length() == 1)
      {
        try
        {
          op = wrapper->_instance->CloseAndSaveToFileAsync();
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }
    
      auto opTask = create_task(op);
      uv_async_t* asyncToken = NodeUtils::Async::GetAsyncToken(args[args.Length() -1].As<Function>());

      opTask.then( [asyncToken] (task<::Windows::Storage::StorageFile^> t) 
      {	
        try
        {
          auto result = t.get();
          NodeUtils::Async::RunCallbackOnMain(asyncToken, [result](NodeUtils::InvokeCallbackDelegate invokeCallback) {


            TryCatch tryCatch;
            Handle<Value> error; 
            Handle<Value> arg1 = NodeRT::Utils::CreateExternalWinRTObject("Windows.Storage", "StorageFile", result);
            if (tryCatch.HasCaught())
            {
              error = tryCatch.Exception()->ToObject();
            }
            else 
            {
              error = Undefined();
            }
            if (arg1.IsEmpty()) arg1 = Undefined();
            Handle<Value> args[] = {error, arg1};

            invokeCallback(_countof(args), args);
          });
        }
        catch (Platform::Exception^ exception)
        {
          NodeUtils::Async::RunCallbackOnMain(asyncToken, [exception](NodeUtils::InvokeCallbackDelegate invokeCallback) {
             
            Handle<Value> error = NodeRT::Utils::WinRtExceptionToJsError(exception);
        
            Handle<Value> args[] = {error};
            invokeCallback(_countof(args), args);
          });
        }  		
      });

      return scope.Close(Undefined());
    }
  
    static Handle<Value> AddLoggingChannel(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::IFileLoggingSession^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      IFileLoggingSession *wrapper = IFileLoggingSession::Unwrap<IFileLoggingSession>(args.This());

      if (args.Length() == 1
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::ILoggingChannel^>(args[0]))
      {
        try
        {
          ::Windows::Foundation::Diagnostics::ILoggingChannel^ arg0 = UnwrapILoggingChannel(args[0]);
          
          wrapper->_instance->AddLoggingChannel(arg0);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else if (args.Length() == 2
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::ILoggingChannel^>(args[0])
        && args[1]->IsInt32())
      {
        try
        {
          ::Windows::Foundation::Diagnostics::ILoggingChannel^ arg0 = UnwrapILoggingChannel(args[0]);
          ::Windows::Foundation::Diagnostics::LoggingLevel arg1 = static_cast<::Windows::Foundation::Diagnostics::LoggingLevel>(args[1]->Int32Value());
          
          wrapper->_instance->AddLoggingChannel(arg0, arg1);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> RemoveLoggingChannel(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::IFileLoggingSession^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      IFileLoggingSession *wrapper = IFileLoggingSession::Unwrap<IFileLoggingSession>(args.This());

      if (args.Length() == 1
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::ILoggingChannel^>(args[0]))
      {
        try
        {
          ::Windows::Foundation::Diagnostics::ILoggingChannel^ arg0 = UnwrapILoggingChannel(args[0]);
          
          wrapper->_instance->RemoveLoggingChannel(arg0);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }



    static Handle<Value> NameGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::IFileLoggingSession^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      IFileLoggingSession *wrapper = IFileLoggingSession::Unwrap<IFileLoggingSession>(info.This());

      try 
      {
        Platform::String^ result = wrapper->_instance->Name;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    


    static v8::Handle<v8::Value> AddListener(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() < 2 || !args[0]->IsString() || !args[1]->IsFunction())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"wrong arguments, expected arguments are eventName(string),callback(function)")));
        return scope.Close(Undefined());
      }

      String::Value eventName(args[0]);
      auto str = *eventName;
      
      Local<Function> callback = args[1].As<Function>();
      
      ::Windows::Foundation::EventRegistrationToken registrationToken;
      if (NodeRT::Utils::CaseInsenstiveEquals(L"logFileGenerated", str))
      {
        if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::IFileLoggingSession^>(args.This()))
        {
          ThrowException(Exception::Error(NodeRT::Utils::NewString(L"The caller of this method isn't of the expected type or internal WinRt object was disposed")));
          return scope.Close(Undefined());
        }
        IFileLoggingSession *wrapper = IFileLoggingSession::Unwrap<IFileLoggingSession>(args.This());
      
        try
        {
          std::shared_ptr<Persistent<Object>> callbackObjPtr(new Persistent<Object>(Persistent<Object>::New(NodeRT::Utils::CreateCallbackObjectInDomain(callback))), 
            [] (Persistent<Object> *ptr ) {
              NodeUtils::Async::RunOnMain([ptr]() {
                ptr->Dispose();
                delete ptr;
            });
          });

          registrationToken = wrapper->_instance->LogFileGenerated::add(
            ref new ::Windows::Foundation::TypedEventHandler<::Windows::Foundation::Diagnostics::IFileLoggingSession^, ::Windows::Foundation::Diagnostics::LogFileGeneratedEventArgs^>(
            [callbackObjPtr](::Windows::Foundation::Diagnostics::IFileLoggingSession^ arg0, ::Windows::Foundation::Diagnostics::LogFileGeneratedEventArgs^ arg1) {
              NodeUtils::Async::RunOnMain([callbackObjPtr , arg0, arg1]() {
                TryCatch tryCatch;
              
                Handle<Value> error;

                Handle<Value> wrappedArg0 = WrapIFileLoggingSession(arg0);
                Handle<Value> wrappedArg1 = WrapLogFileGeneratedEventArgs(arg1);

                if (tryCatch.HasCaught())
                {
                  error = tryCatch.Exception()->ToObject();
                }
                else 
                {
                  error = Undefined();
                }


                if (wrappedArg0.IsEmpty()) wrappedArg0 = Undefined();
                if (wrappedArg1.IsEmpty()) wrappedArg1 = Undefined();

                Handle<Value> args[] = { wrappedArg0, wrappedArg1 };
                NodeRT::Utils::CallCallbackInDomain(*callbackObjPtr, _countof(args), args);
              });
            })
          );
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }

      }
      else 
      {
        ThrowException(Exception::Error(String::Concat(NodeRT::Utils::NewString(L"given event name isn't supported: "), args[0].As<String>())));
        return scope.Close(Undefined());
      }

      Local<Value> tokenMap = callback->GetHiddenValue(String::NewSymbol(REGISTRATION_TOKEN_MAP_PROPERTY_NAME));
                
      if (tokenMap.IsEmpty() || tokenMap->Equals(Undefined()))
      {
          tokenMap = Object::New();
          callback->SetHiddenValue(String::NewSymbol(REGISTRATION_TOKEN_MAP_PROPERTY_NAME), tokenMap);
      }

      tokenMap.As<Object>()->Set(args[1], CreateOpaqueWrapper(::Windows::Foundation::PropertyValue::CreateInt64(registrationToken.Value)));
                
      return scope.Close(Undefined());
    }

    static v8::Handle<v8::Value> RemoveListener(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() < 2 || !args[0]->IsString() || !args[1]->IsFunction())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"wrong arguments, expected a string and a callback")));
        return scope.Close(Undefined());
      }

      String::Value eventName(args[0]);
      auto str = *eventName;

      if ((NodeRT::Utils::CaseInsenstiveEquals(L"logFileGenerated", str)))
      {
        ThrowException(Exception::Error(String::Concat(NodeRT::Utils::NewString(L"given event name isn't supported: "), args[0].As<String>())));
        return scope.Close(Undefined());
      }

      Local<Function> callback = args[1].As<Function>();
      Handle<Value> tokenMap = callback->GetHiddenValue(String::NewSymbol(REGISTRATION_TOKEN_MAP_PROPERTY_NAME));
                
      if (tokenMap.IsEmpty() || tokenMap->Equals(Undefined()))
      {
        return scope.Close(Undefined());
      }

      Handle<Value> opaqueWrapperObj =  tokenMap.As<Object>()->Get(args[1]);

      if (opaqueWrapperObj.IsEmpty() || opaqueWrapperObj->Equals(Undefined()))
      {
        return scope.Close(Undefined());
      }

      OpaqueWrapper *opaqueWrapper = OpaqueWrapper::Unwrap<OpaqueWrapper>(opaqueWrapperObj.As<Object>());
            
      long long tokenValue = (long long) opaqueWrapper->GetObjectInstance();
      ::Windows::Foundation::EventRegistrationToken registrationToken;
      registrationToken.Value = tokenValue;
        
      try 
      {
        if (NodeRT::Utils::CaseInsenstiveEquals(L"logFileGenerated", str))
        {
          if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::IFileLoggingSession^>(args.This()))
          {
            ThrowException(Exception::Error(NodeRT::Utils::NewString(L"The caller of this method isn't of the expected type or internal WinRt object was disposed")));
            return scope.Close(Undefined());
          }
          IFileLoggingSession *wrapper = IFileLoggingSession::Unwrap<IFileLoggingSession>(args.This());
          wrapper->_instance->LogFileGenerated::remove(registrationToken);
        }
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }

      tokenMap.As<Object>()->Delete(args[0].As<String>());

      return scope.Close(Undefined());
    }
  private:
    ::Windows::Foundation::Diagnostics::IFileLoggingSession^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapIFileLoggingSession(::Windows::Foundation::Diagnostics::IFileLoggingSession^ wintRtInstance);
    friend ::Windows::Foundation::Diagnostics::IFileLoggingSession^ UnwrapIFileLoggingSession(Handle<Value> value);
    friend bool IsIFileLoggingSessionWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> IFileLoggingSession::s_constructorTemplate;

  v8::Handle<v8::Value> WrapIFileLoggingSession(::Windows::Foundation::Diagnostics::IFileLoggingSession^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(IFileLoggingSession::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Foundation::Diagnostics::IFileLoggingSession^ UnwrapIFileLoggingSession(Handle<Value> value)
  {
     return IFileLoggingSession::Unwrap<IFileLoggingSession>(value.As<Object>())->_instance;
  }

  void InitIFileLoggingSession(Handle<Object> exports)
  {
    IFileLoggingSession::Init(exports);
  }

  class FileLoggingSession : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("FileLoggingSession"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
      Handle<Value> asyncSymbol = String::NewSymbol("__winRtAsync__");
      Handle<Function> func;
            
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("addLoggingChannel"), FunctionTemplate::New(AddLoggingChannel)->GetFunction());
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("removeLoggingChannel"), FunctionTemplate::New(RemoveLoggingChannel)->GetFunction());
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("close"), FunctionTemplate::New(Close)->GetFunction());
      
            
      func = FunctionTemplate::New(CloseAndSaveToFileAsync)->GetFunction();
      func->Set(asyncSymbol, True(), PropertyAttribute::DontEnum);
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("closeAndSaveToFileAsync"), func);
      
            
      Local<Function> addListenerFunc = FunctionTemplate::New(AddListener)->GetFunction();
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("addListener"), addListenerFunc);
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("on"), addListenerFunc);
      Local<Function> removeListenerFunc = FunctionTemplate::New(RemoveListener)->GetFunction();
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("removeListener"), removeListenerFunc);
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("off"), removeListenerFunc);
            
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("name"), NameGetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("FileLoggingSession"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    FileLoggingSession(::Windows::Foundation::Diagnostics::FileLoggingSession^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Foundation::Diagnostics::FileLoggingSession^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::FileLoggingSession^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Foundation::Diagnostics::FileLoggingSession^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else if (args.Length() == 1
        && args[0]->IsString())
      {
        try
        {
          Platform::String^ arg0 = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(args[0])));
          
          winRtInstance = ref new ::Windows::Foundation::Diagnostics::FileLoggingSession(arg0);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      FileLoggingSession *wrapperInstance = new FileLoggingSession(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


    static Handle<Value> CloseAndSaveToFileAsync(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::FileLoggingSession^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      if (args.Length() == 0 || !args[args.Length() -1]->IsFunction())
      {
          ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: No callback was given")));
          return scope.Close(Undefined());
      }

      FileLoggingSession *wrapper = FileLoggingSession::Unwrap<FileLoggingSession>(args.This());

      ::Windows::Foundation::IAsyncOperation<::Windows::Storage::StorageFile^>^ op;
    

      if (args.Length() == 1)
      {
        try
        {
          op = wrapper->_instance->CloseAndSaveToFileAsync();
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }
    
      auto opTask = create_task(op);
      uv_async_t* asyncToken = NodeUtils::Async::GetAsyncToken(args[args.Length() -1].As<Function>());

      opTask.then( [asyncToken] (task<::Windows::Storage::StorageFile^> t) 
      {	
        try
        {
          auto result = t.get();
          NodeUtils::Async::RunCallbackOnMain(asyncToken, [result](NodeUtils::InvokeCallbackDelegate invokeCallback) {


            TryCatch tryCatch;
            Handle<Value> error; 
            Handle<Value> arg1 = NodeRT::Utils::CreateExternalWinRTObject("Windows.Storage", "StorageFile", result);
            if (tryCatch.HasCaught())
            {
              error = tryCatch.Exception()->ToObject();
            }
            else 
            {
              error = Undefined();
            }
            if (arg1.IsEmpty()) arg1 = Undefined();
            Handle<Value> args[] = {error, arg1};

            invokeCallback(_countof(args), args);
          });
        }
        catch (Platform::Exception^ exception)
        {
          NodeUtils::Async::RunCallbackOnMain(asyncToken, [exception](NodeUtils::InvokeCallbackDelegate invokeCallback) {
             
            Handle<Value> error = NodeRT::Utils::WinRtExceptionToJsError(exception);
        
            Handle<Value> args[] = {error};
            invokeCallback(_countof(args), args);
          });
        }  		
      });

      return scope.Close(Undefined());
    }
  
    static Handle<Value> AddLoggingChannel(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::FileLoggingSession^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      FileLoggingSession *wrapper = FileLoggingSession::Unwrap<FileLoggingSession>(args.This());

      if (args.Length() == 1
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::ILoggingChannel^>(args[0]))
      {
        try
        {
          ::Windows::Foundation::Diagnostics::ILoggingChannel^ arg0 = UnwrapILoggingChannel(args[0]);
          
          wrapper->_instance->AddLoggingChannel(arg0);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else if (args.Length() == 2
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::ILoggingChannel^>(args[0])
        && args[1]->IsInt32())
      {
        try
        {
          ::Windows::Foundation::Diagnostics::ILoggingChannel^ arg0 = UnwrapILoggingChannel(args[0]);
          ::Windows::Foundation::Diagnostics::LoggingLevel arg1 = static_cast<::Windows::Foundation::Diagnostics::LoggingLevel>(args[1]->Int32Value());
          
          wrapper->_instance->AddLoggingChannel(arg0, arg1);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> RemoveLoggingChannel(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::FileLoggingSession^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      FileLoggingSession *wrapper = FileLoggingSession::Unwrap<FileLoggingSession>(args.This());

      if (args.Length() == 1
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::ILoggingChannel^>(args[0]))
      {
        try
        {
          ::Windows::Foundation::Diagnostics::ILoggingChannel^ arg0 = UnwrapILoggingChannel(args[0]);
          
          wrapper->_instance->RemoveLoggingChannel(arg0);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> Close(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::FileLoggingSession^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      FileLoggingSession *wrapper = FileLoggingSession::Unwrap<FileLoggingSession>(args.This());

      if (args.Length() == 0)
      {
        try
        {
          delete wrapper->_instance;
          wrapper->_instance = nullptr;
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }




    static Handle<Value> NameGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::FileLoggingSession^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      FileLoggingSession *wrapper = FileLoggingSession::Unwrap<FileLoggingSession>(info.This());

      try 
      {
        Platform::String^ result = wrapper->_instance->Name;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    


    static v8::Handle<v8::Value> AddListener(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() < 2 || !args[0]->IsString() || !args[1]->IsFunction())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"wrong arguments, expected arguments are eventName(string),callback(function)")));
        return scope.Close(Undefined());
      }

      String::Value eventName(args[0]);
      auto str = *eventName;
      
      Local<Function> callback = args[1].As<Function>();
      
      ::Windows::Foundation::EventRegistrationToken registrationToken;
      if (NodeRT::Utils::CaseInsenstiveEquals(L"logFileGenerated", str))
      {
        if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::FileLoggingSession^>(args.This()))
        {
          ThrowException(Exception::Error(NodeRT::Utils::NewString(L"The caller of this method isn't of the expected type or internal WinRt object was disposed")));
          return scope.Close(Undefined());
        }
        FileLoggingSession *wrapper = FileLoggingSession::Unwrap<FileLoggingSession>(args.This());
      
        try
        {
          std::shared_ptr<Persistent<Object>> callbackObjPtr(new Persistent<Object>(Persistent<Object>::New(NodeRT::Utils::CreateCallbackObjectInDomain(callback))), 
            [] (Persistent<Object> *ptr ) {
              NodeUtils::Async::RunOnMain([ptr]() {
                ptr->Dispose();
                delete ptr;
            });
          });

          registrationToken = wrapper->_instance->LogFileGenerated::add(
            ref new ::Windows::Foundation::TypedEventHandler<::Windows::Foundation::Diagnostics::IFileLoggingSession^, ::Windows::Foundation::Diagnostics::LogFileGeneratedEventArgs^>(
            [callbackObjPtr](::Windows::Foundation::Diagnostics::IFileLoggingSession^ arg0, ::Windows::Foundation::Diagnostics::LogFileGeneratedEventArgs^ arg1) {
              NodeUtils::Async::RunOnMain([callbackObjPtr , arg0, arg1]() {
                TryCatch tryCatch;
              
                Handle<Value> error;

                Handle<Value> wrappedArg0 = WrapIFileLoggingSession(arg0);
                Handle<Value> wrappedArg1 = WrapLogFileGeneratedEventArgs(arg1);

                if (tryCatch.HasCaught())
                {
                  error = tryCatch.Exception()->ToObject();
                }
                else 
                {
                  error = Undefined();
                }


                if (wrappedArg0.IsEmpty()) wrappedArg0 = Undefined();
                if (wrappedArg1.IsEmpty()) wrappedArg1 = Undefined();

                Handle<Value> args[] = { wrappedArg0, wrappedArg1 };
                NodeRT::Utils::CallCallbackInDomain(*callbackObjPtr, _countof(args), args);
              });
            })
          );
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }

      }
      else 
      {
        ThrowException(Exception::Error(String::Concat(NodeRT::Utils::NewString(L"given event name isn't supported: "), args[0].As<String>())));
        return scope.Close(Undefined());
      }

      Local<Value> tokenMap = callback->GetHiddenValue(String::NewSymbol(REGISTRATION_TOKEN_MAP_PROPERTY_NAME));
                
      if (tokenMap.IsEmpty() || tokenMap->Equals(Undefined()))
      {
          tokenMap = Object::New();
          callback->SetHiddenValue(String::NewSymbol(REGISTRATION_TOKEN_MAP_PROPERTY_NAME), tokenMap);
      }

      tokenMap.As<Object>()->Set(args[1], CreateOpaqueWrapper(::Windows::Foundation::PropertyValue::CreateInt64(registrationToken.Value)));
                
      return scope.Close(Undefined());
    }

    static v8::Handle<v8::Value> RemoveListener(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() < 2 || !args[0]->IsString() || !args[1]->IsFunction())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"wrong arguments, expected a string and a callback")));
        return scope.Close(Undefined());
      }

      String::Value eventName(args[0]);
      auto str = *eventName;

      if ((NodeRT::Utils::CaseInsenstiveEquals(L"logFileGenerated", str)))
      {
        ThrowException(Exception::Error(String::Concat(NodeRT::Utils::NewString(L"given event name isn't supported: "), args[0].As<String>())));
        return scope.Close(Undefined());
      }

      Local<Function> callback = args[1].As<Function>();
      Handle<Value> tokenMap = callback->GetHiddenValue(String::NewSymbol(REGISTRATION_TOKEN_MAP_PROPERTY_NAME));
                
      if (tokenMap.IsEmpty() || tokenMap->Equals(Undefined()))
      {
        return scope.Close(Undefined());
      }

      Handle<Value> opaqueWrapperObj =  tokenMap.As<Object>()->Get(args[1]);

      if (opaqueWrapperObj.IsEmpty() || opaqueWrapperObj->Equals(Undefined()))
      {
        return scope.Close(Undefined());
      }

      OpaqueWrapper *opaqueWrapper = OpaqueWrapper::Unwrap<OpaqueWrapper>(opaqueWrapperObj.As<Object>());
            
      long long tokenValue = (long long) opaqueWrapper->GetObjectInstance();
      ::Windows::Foundation::EventRegistrationToken registrationToken;
      registrationToken.Value = tokenValue;
        
      try 
      {
        if (NodeRT::Utils::CaseInsenstiveEquals(L"logFileGenerated", str))
        {
          if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Diagnostics::FileLoggingSession^>(args.This()))
          {
            ThrowException(Exception::Error(NodeRT::Utils::NewString(L"The caller of this method isn't of the expected type or internal WinRt object was disposed")));
            return scope.Close(Undefined());
          }
          FileLoggingSession *wrapper = FileLoggingSession::Unwrap<FileLoggingSession>(args.This());
          wrapper->_instance->LogFileGenerated::remove(registrationToken);
        }
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }

      tokenMap.As<Object>()->Delete(args[0].As<String>());

      return scope.Close(Undefined());
    }
  private:
    ::Windows::Foundation::Diagnostics::FileLoggingSession^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapFileLoggingSession(::Windows::Foundation::Diagnostics::FileLoggingSession^ wintRtInstance);
    friend ::Windows::Foundation::Diagnostics::FileLoggingSession^ UnwrapFileLoggingSession(Handle<Value> value);
    friend bool IsFileLoggingSessionWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> FileLoggingSession::s_constructorTemplate;

  v8::Handle<v8::Value> WrapFileLoggingSession(::Windows::Foundation::Diagnostics::FileLoggingSession^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(FileLoggingSession::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Foundation::Diagnostics::FileLoggingSession^ UnwrapFileLoggingSession(Handle<Value> value)
  {
     return FileLoggingSession::Unwrap<FileLoggingSession>(value.As<Object>())->_instance;
  }

  void InitFileLoggingSession(Handle<Object> exports)
  {
    FileLoggingSession::Init(exports);
  }

} } } } 

void init(Handle<Object> exports)
{
  if (FAILED(CoInitializeEx(nullptr, COINIT_MULTITHREADED)))
  {
    ThrowException(v8::Exception::Error(NodeRT::Utils::NewString(L"error in CoInitializeEx()")));
    return;
  }
  
  NodeRT::Windows::Foundation::Diagnostics::InitCausalityTraceLevelEnum(exports);
  NodeRT::Windows::Foundation::Diagnostics::InitCausalitySourceEnum(exports);
  NodeRT::Windows::Foundation::Diagnostics::InitCausalityRelationEnum(exports);
  NodeRT::Windows::Foundation::Diagnostics::InitCausalitySynchronousWorkEnum(exports);
  NodeRT::Windows::Foundation::Diagnostics::InitErrorOptionsEnum(exports);
  NodeRT::Windows::Foundation::Diagnostics::InitLoggingLevelEnum(exports);
  NodeRT::Windows::Foundation::Diagnostics::InitTracingStatusChangedEventArgs(exports);
  NodeRT::Windows::Foundation::Diagnostics::InitAsyncCausalityTracer(exports);
  NodeRT::Windows::Foundation::Diagnostics::InitIErrorReportingSettings(exports);
  NodeRT::Windows::Foundation::Diagnostics::InitRuntimeBrokerErrorSettings(exports);
  NodeRT::Windows::Foundation::Diagnostics::InitILoggingChannel(exports);
  NodeRT::Windows::Foundation::Diagnostics::InitLoggingChannel(exports);
  NodeRT::Windows::Foundation::Diagnostics::InitLoggingActivity(exports);
  NodeRT::Windows::Foundation::Diagnostics::InitILoggingSession(exports);
  NodeRT::Windows::Foundation::Diagnostics::InitLoggingSession(exports);
  NodeRT::Windows::Foundation::Diagnostics::InitLogFileGeneratedEventArgs(exports);
  NodeRT::Windows::Foundation::Diagnostics::InitIFileLoggingSession(exports);
  NodeRT::Windows::Foundation::Diagnostics::InitFileLoggingSession(exports);

  NodeRT::Utils::RegisterNameSpace("Windows.Foundation.Diagnostics", exports);
}


NODE_MODULE(binding, init)